package com.wawrzek.fumbo;

import jade.content.lang.*;
import jade.content.lang.sl.*;
import jade.content.onto.*;
import jade.core.*;
import jade.lang.acl.*;

import java.io.*;

import org.apache.log4j.*;

import com.wawrzek.fumbo.core.*;
import com.wawrzek.fumbo.tools.*;
import com.wawrzek.fumbo.tools.basic.*;
import com.wawrzek.fumbo.worker.*;

/**
 * Main class for worker agent. Worker agent does all the work with result set 
 * 
 * @author Wawrzek
 *  
 */
public class Worker extends Agent
{
    private Record config;
    private Codec codec = new SLCodec();
    private Ontology ontology = InternalOntology.getInstance();
    private Rule rule;
    private Logger logger;
    private AID coordinatorAID;

    /**
     * Setups worker agent - sets its behaviourc etc.
     */
    protected void setup()
    {
        try
        {
            setupOut();
            Object[] args = getArguments();
            if (args.length != 2)
                throw new AgentException("Bad number of arguments: should be two");
            coordinatorAID = (AID) (args[1]);
            startConfig();
            getContentManager().registerLanguage(codec);
            getContentManager().registerOntology(ontology);
            getContentManager().setValidationMode(true);
            rule = new Rule((String) args[0]);
            setBehaviours();
            logger.info(getAID().getName() + " is ready.");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            logger.fatal("Abnormal agent termination: " + ex.getMessage());
            sendDeathInfo();
        }
    }

    /**
     * Sends info about suicide plans to coordinator. Then, agent is killed by coordinator.
     *
     */
    private void sendDeathInfo()
    {
        Logger.getLogger(Consts.WORKER_LOG).info("sending failure info to coordianator");
        ACLMessage msg = new ACLMessage(ACLMessage.FAILURE);
        msg.addReceiver(coordinatorAID);
        msg.setContent("killme");
        this.send(msg);
    }

    /**
     * Prepares standard output and logging
     * 
     * @throws FileNotFoundException
     */
    private void setupOut() throws FileNotFoundException
    {
        PropertyConfigurator.configure("cfg/logging.cfg");
        logger = Logger.getLogger(Consts.WORKER_LOG);

        PrintStream out = new PrintStream(new FileOutputStream("out.txt"));
        System.setOut(out);
        System.setErr(out);
    }

    /**
     * Reads config from a file
     * 
     * @throws AgentException
     */
    private void startConfig()
    {
        config = new Record();
        config.read("cfg" + File.separator + "fumbo.cfg");
    }

    /**
     * Sets all worker behaviours
     *
     */
    private void setBehaviours()
    {
        addBehaviour(new TaskRequestLoop());
    }

    /**
     * Does all the work with closing an agent
     */
    protected void takeDown()
    {
        Logger.getLogger(Consts.WORKER_LOG).info("bye bye");
    }

    /**
     * returns worker's configoration
     * @return worker's configoration
     */
    public Record getConfig()
    {
        return config;
    }

    /**
     * sets worker configuration
     * @param config configuration to be set
     */
    public void setConfig(Record config)
    {
        this.config = config;
    }

    public Codec getCodec()
    {
        return codec;
    }

    public Ontology getOntology()
    {
        return ontology;
    }

    public Rule getRule()
    {
        return rule;
    }
}