package com.wawrzek.fumbo.coordinator;

import jade.content.lang.Codec.*;
import jade.content.onto.*;
import jade.content.onto.basic.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

import org.apache.log4j.*;

import com.wawrzek.fumbo.*;
import com.wawrzek.fumbo.core.*;
import com.wawrzek.fumbo.tools.*;

/**
 * Behaviour of getting new task (new query) from outside the Fumbo. ExternalOntology is used to translate
 * message content into Task object.
 * 
 * @author Wawrzek
 *  
 */

public class NewTaskBehaviour extends OneShotBehaviour
{
    /** message that was received */
    private ACLMessage msg;

    /**
     * @param msg received message
     */
    public NewTaskBehaviour(ACLMessage msg)
    {
        super();
        this.msg = msg;
    }

    /* (non-Javadoc)
     * @see jade.core.behaviours.Behaviour#action()
     */
    public void action()
    {
        try
        {
            Logger.getLogger(Consts.COORDINATOR_LOG).info("new external task received.");
            TaskManager tm = ((Coordinator) myAgent).getTaskManager();
        
            Action a = (Action) myAgent.getContentManager().extractContent(msg);
            ExternalTask etask = (ExternalTask) a.getAction();
            
            

            long taskId = tm.registerTask(etask, ((Coordinator) myAgent).getAgentsManager().getList(), msg);
            tm.runTask(taskId, (Coordinator)myAgent);
        }
        catch (CodecException ce)
        {
            ce.printStackTrace();
            Logger.getLogger(Consts.COORDINATOR_LOG).error("codec error while handling a message " + msg);
        }
        catch (OntologyException oe)
        {
            oe.printStackTrace();
            Logger.getLogger(Consts.COORDINATOR_LOG).error("ontology error while handling a message " + msg);
        }
        catch (AgentException e)
        {
            e.printStackTrace();
            Logger.getLogger(Consts.COORDINATOR_LOG).error("error while handling a message " + msg);
        }
    }
}