package com.wawrzek.fumbo.core;

import jade.content.onto.*;
import jade.content.schema.*;

import org.apache.log4j.*;

import com.wawrzek.fumbo.*;
import com.wawrzek.fumbo.coordinator.*;

/**
 * @author wawrzek
 * 
 * The ontology used in communication between coodinator and workers
 */
public class InternalOntology extends Ontology
{
    public static final String ONTOLOGY_NAME = "tuning-ontology";
    private static Ontology theInstance = new InternalOntology();

    private static final String TASK = "Task";
    private static final String TASK_ID = "id";
    private static final String TASK_RDQL_QUESTION = "rdqlQuestion";
    private static final String TASK_RESULTSET = "resultSet";
    private static final String TASK_AGENTS_RING= "agentsRing";
    private static final String TASK_CURRENT_AGENT = "current";
    private static final String TASK_COORDINATOR_AID = "coordinator";
    private static final String TASK_RESULTS_COUNT = "resultsCount";
    
    
    private static final String WORKER_INFO = "workerInfo";
    private static final String WORKER_INFO_NAME = "name";
    private static final String WORKER_INFO_AID = "aid";

    public static Ontology getInstance()
    {
        return theInstance;
    }

    private InternalOntology()
    {
        super(ONTOLOGY_NAME, BasicOntology.getInstance());
        try
        {
            add(new AgentActionSchema(TASK), Task.class);
            add(new ConceptSchema(WORKER_INFO), WorkerInfo.class);

            ConceptSchema cs = (ConceptSchema)getSchema(WORKER_INFO);
            cs.add(WORKER_INFO_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            cs.add(WORKER_INFO_AID, (ConceptSchema) getSchema(BasicOntology.AID));
            
            AgentActionSchema aas = (AgentActionSchema) getSchema(TASK);
            aas.add(TASK_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
            aas.add(TASK_COORDINATOR_AID, (ConceptSchema) getSchema(BasicOntology.AID));
            aas.add(TASK_CURRENT_AGENT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
            aas.add(TASK_RESULTS_COUNT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
            aas.add(TASK_RDQL_QUESTION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            aas.add(TASK_RESULTSET, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            aas.add(TASK_AGENTS_RING, (ConceptSchema) getSchema(WORKER_INFO), 1,ObjectSchema.UNLIMITED);
        }
        catch (OntologyException ex)
        {
            Logger.getLogger(Consts.COORDINATOR_LOG).fatal("error while creating internal ontology");
            ex.printStackTrace();
        }
    }
}