package com.wawrzek.fumbo.core;

import org.apache.log4j.*;

import jade.content.*;
import jade.core.*;
import jade.lang.acl.*;
import jade.util.leap.*;

import com.wawrzek.fumbo.*;
import com.wawrzek.fumbo.coordinator.*;
import com.wawrzek.fumbo.tools.*;

/**
 * Class providing one internal task that coordinator sends to workers
 * @author Wawrzek
 */

public class Task implements AgentAction
{
    /** base RDQL question */
    private String rdqlQuestion;
    /** result set that is built */
    private String resultSet;
    /** number of a task */
    private long id;
    /** list of workers that must work over a task */
    private List agentsRing;
    /** number of worker that works currently over the task*/
    private int current;
    /** AID of coordinator that made a task and that waits for an answer */
    private AID coordinator;
    /** base query that will be used to make a reply message */
    ACLMessage query;
    /** number of current answers*/ 
    private int resultsCount;

    /**
     * For coordinator - only to create a task 
     */
    public Task(ExternalTask etask, long id, ACLMessage query)
    {
        this.id = id;
        this.query = query;
        this.rdqlQuestion = etask.getQuestion();
        this.resultSet = etask.getResultSet();
        this.resultsCount = (int) etask.getResultCount();
        current = 0;
    }

    public Task()
    {
    }

    /**
     * Does the core job in the System. Expands base result set upon a rule.
     * 
     * @param rule rule according to which result set will be expanded
     * @param dataURL url to data source
     * @return expanded result set
     * @throws AgentException
     */
    public void expandResultSet(Rule rule, String dataURL, String baseUri) throws AgentException
    {
        if (!rule.isTrue(rdqlQuestion, resultSet, baseUri))
            return;

        RdfEngine engine = new RdfEngine();
        
        engine.readDataFromURL(dataURL);

        engine.query(rule.getQuery(rdqlQuestion));
        engine.outerJoin(resultSet, baseUri);
        resultSet = engine.getAnswer();
        resultsCount = engine.getAnswerCount();
        engine.close();
    }

    public void increaseRing()
    {
        current++;
    }

    /**
     * Returns the next agent that the task will be send to.
     * @return
     */
    public WorkerInfo getNextAgent()
    {
        return (WorkerInfo) agentsRing.get(current);
    }

    //getters and setter to implement Concept:
    public long getId()
    {
        return id;
    }

    public void setId(long id)
    {
        this.id = id;
    }

    public String getRdqlQuestion()
    {
        return rdqlQuestion;
    }

    public void setRdqlQuestion(String rdqlQuestion)
    {
        this.rdqlQuestion = rdqlQuestion;
    }

    public String getResultSet()
    {
        return resultSet;
    }

    public void setResultSet(String resultSet)
    {
        this.resultSet = resultSet;
    }

    public int getCurrent()
    {
        return current;
    }

    public void setCurrent(int current)
    {
        this.current = current;
    }

    public List getAgentsRing()
    {
        return agentsRing;
    }

    public void setAgentsRing(List agentsRing)
    {
        this.agentsRing = agentsRing;
    }

    public AID getCoordinator()
    {
        return coordinator;
    }

    public void setCoordinator(AID aid)
    {
        this.coordinator = aid;
    }

    public boolean isReady()
    {
        return (current == (agentsRing.size()));
    }

    public ACLMessage getMessage()
    {
        return query;
    }
    
    public int getResultsCount()
    {
        return resultsCount;
    }
    public void setResultsCount(int resultsCount)
    {
        this.resultsCount = resultsCount;
    }
}