package com.wawrzek.fumbo.core.rulepart;

import com.wawrzek.fumbo.tools.*;

/**
 * @author Wawrzek
 * 
 * Parser for conditions in rule files.
 */
public class ConditionParser
{
    /**
     * parses condition part of rule (between IF and THEN)
     * @param src string to be parsed
     * @return parsed Condition
     * @throws AgentException in case of parse errors
     */
    public static Condition parse(String src) throws AgentException
    {
        Condition result = null;
        if (src.startsWith("COUNT"))
            return checkCount(src);
        if (src.equals("TRUE"))
            return new TrueCondition(src);
        if (src.equals("FALSE"))
            return new FalseCondition(src);
        throw new AgentException("Syntax error in condition " + src + ". Shoul start with one of (TRUE, FALSE, COUNT)");
    }

    /**
     * Tries to parse string as COUNT condition
     * @param src string to be parsed
     * @return parsed condition
     * @throws AgentException in case of parse problems
     */
    private static CountCondition checkCount(String src) throws AgentException
    {
        src = src.substring(5, src.length());
        int type = -1;
        int prefixCount = 1;
        src = src.trim();

        if (src.startsWith("="))
            type = CountCondition.EQUAL;
        else if (src.startsWith("<="))
        {
            prefixCount = 2;
            type = CountCondition.LESSOREQUAL;
        }
        else if (src.startsWith("<"))
            type = CountCondition.LESS;
        else if (src.startsWith(">="))
        {
            prefixCount = 2;
            type = CountCondition.GRATEROREQUAL;
        }
        else if (src.startsWith(">"))
            type = CountCondition.GRATER;
        else
            throw new AgentException("error parsing CountCondition");

        src = src.substring(prefixCount, src.length());
        src = src.trim();
        return new CountCondition(src, type, (new Integer(src)).intValue());
    }

}