package com.wawrzek.fumbo.core.rulepart;


import com.wawrzek.fumbo.core.*;
import com.wawrzek.fumbo.tools.*;

/**
 * Condition that is fulfilled, if result count is in some relation with specified value. 
 * Can check if count is LESS, LESSOREQUAL, EQUAL, GREATEROREQUAL, GREATER or NOTEQUAL than a value
 *  
 * @author Wawrzek
 */
public class CountCondition extends Condition
{
    public static final int LESS = 0;
    public static final int LESSOREQUAL = 1;
    public static final int EQUAL = 2;
    public static final int GRATEROREQUAL = 3;
    public static final int GRATER = 4;
    public static final int NOTEQUAL = 5;

    /** number that count will be compared to */
    private int number;
    /** type of relation to be fulfilled (LESS, LESSOREQUAL, EQUAL, GREATEROREQUAL, GREATER, NOTEQUAL)*/ 
    private int type;
    
    public CountCondition(String source, int type, int number)
    {
        super(source);
        this.type = type;
        this.number = number;
    }

    /* (non-Javadoc)
     * @see com.wawrzek.fumbo.core.rulepart.Condition#isTrue(java.lang.String, java.lang.String)
     */
    public boolean isTrue(String question, String answer, String baseUri) throws AgentException
    {
        int x = getCount(answer, baseUri);
        switch (type) {
            case LESS:
                return (x < number);
            case LESSOREQUAL:
                return (x <= number);
            case EQUAL:
                return (x == number);
            case GRATEROREQUAL:
                return (x >= number);
            case GRATER:
                return (x > number);
            case NOTEQUAL:
                return (x != number);
        }
        throw new AgentException("Strange count condition type: "+type);
    }

    /**
     * Returns number of responses in specified RDF
     * @param rdf RDF to be counted
     * @return number of responses in specified RDF
     */
    private int getCount(String rdf, String baseUri)
    {
        RdfEngine engine = new RdfEngine();
        engine.readDataFromString(rdf, baseUri);
        int result = (int) engine.getDataCount();
        engine.close();
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        String res = "Condtion: count";
        switch (type) {
            case LESS:
                res += "<";
                break;
            case LESSOREQUAL:
                res += "<=";
                break;
            case EQUAL:
                res += "=";
                break;
            case GRATEROREQUAL:
                res += ">=";
                break;
            case GRATER:
                res += ">";
                break;
        }
        res += number;
        return res;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o)
    {
        if (!(o instanceof CountCondition))
            return false;
        CountCondition another = (CountCondition) o;
        return ((this.type == another.type) && (this.number == another.number));
    }
}