package com.wawrzek.fumbo.core.rulepart;

import org.apache.log4j.*;

import com.wawrzek.fumbo.*;
import com.wawrzek.fumbo.core.*;
import com.wawrzek.fumbo.tools.*;

/**
 * @author wawrzek
 * 
 * Class providing ignoration of some part of RDQL query
 */
public class Ignoration
{
    /** value that will be ignored (URI) */
    private String whatToIgnore;
    /** type of ignoration (SUBJECT, PREDICATE, OBJECT */
    private int whatPartIgnore = -1;

    public static final int SUBJECT = 0;
    public static final int PREDICATE = 1;
    public static final int OBJECT = 2;

    /**
     * Builds ignoration upon COMMAND state in rule 
     * @param src COMMAND state (after THEN)
     * @throws AgentException in case of parsing problems
     */
    public Ignoration(String src) throws AgentException
    {
        int commaIndex = src.indexOf(",");
        whatToIgnore = src.substring(commaIndex + 1, src.length()).trim();
        String part = src.substring(0, commaIndex).trim();
        if (part.equals("SUBJECT"))
            whatPartIgnore = SUBJECT;
        else if (part.equals("PREDICATE"))
            whatPartIgnore = PREDICATE;
        else if (part.equals("OBJECT"))
            whatPartIgnore = OBJECT;
        else
            throw new AgentException("error while parsing Ignoration");
    }

    public Ignoration(String whatToIgnore, int whatPartIgnore)
    {
        this.whatPartIgnore = whatPartIgnore;
        this.whatToIgnore = whatToIgnore;
    }

    /**
     * Changes the query  
     * @param src RDQL query to be changed
     * @return changed RDQL query
     */
    public String changeQuery(String src)
    {
        Logger.getLogger(Consts.WORKER_LOG).debug("changing query " + src);
        String result = src;
        RdfEngine engine = new RdfEngine();
        result = engine.ignoreConstraints(src, whatPartIgnore, whatToIgnore);
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o)
    {
        if (!(o instanceof Ignoration))
            return false;
        Ignoration another = (Ignoration) o;
        return ((this.whatToIgnore.equals(another.whatToIgnore)) && (this.whatPartIgnore == another.whatPartIgnore));
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        String s = "Ignore ";
        switch (whatPartIgnore) {
            case SUBJECT:
                s += "SUBJ";
                break;
            case PREDICATE:
                s += "PRED";
                break;
            case OBJECT:
                s += "OBJ";
                break;
        }
        s += " " + whatToIgnore;
        return s;
    }

}