package com.wawrzek.fumbo.tests;

import java.io.*;
import java.util.*;

import junit.framework.*;

import com.wawrzek.fumbo.core.*;
import com.wawrzek.fumbo.core.rulepart.*;

/**
 * @author Wawrzek
 * 
 * Tests for rules - tests creating new Rule, parsing Rule from string.
 */
public class RuleTest extends TestCase
{
    ArrayList rules;
    ArrayList conds;
    ArrayList meanings;
    ArrayList ignores;

    public static void main(String[] args)
    {
        junit.swingui.TestRunner.run(RuleTest.class);
    }

    protected void setUp() throws Exception
    {
        super.setUp();

        rules = new ArrayList();
        meanings = new ArrayList();
        ignores = new ArrayList();
        conds = new ArrayList();

        makeExamples();

    }

    private void makeExamples()
    {
        rules.add("IF FALSE THEN " + "RDQL" + "(SELECT ?restaurant-uri ?restaurant-name " + "	WHERE (?restaurant-uri  <http://purl.org/dc/elements/1.0#Title>  \"Embassy Cafe\"),	" + "         (?restaurant-uri  <http://purl.org/dc/elements/1.0#Title>  ?restaurant-name)	)");
        conds.add(new FalseCondition(null));
        meanings.add(null);
        ignores.add(null);

        rules.add("IF TRUE THEN " + "RDQL" + "(SELECT ?restaurant-uri ?restaurant-name " + "	WHERE (?restaurant-uri  <http://purl.org/dc/elements/1.0#Title>  \"Embassy Cafe\"),	" + "         (?restaurant-uri  <http://purl.org/dc/elements/1.0#Title>  ?restaurant-name)	)");
        conds.add(new TrueCondition(null));
        meanings.add(null);
        ignores.add(null);

        rules.add("IF COUNT   <= 34 THEN " + "RDQL" + "(SELECT ?restaurant-uri ?restaurant-name " + "	WHERE (?restaurant-uri  <http://purl.org/dc/elements/1.0#Title>  \"Embassy Cafe\"),	" + "         (?restaurant-uri  <http://purl.org/dc/elements/1.0#Title>  ?restaurant-name)	)");
        conds.add(new CountCondition(null, CountCondition.LESSOREQUAL, 34));
        meanings.add(null);
        ignores.add(null);

                
        rules.add("IF   COUNT   >= 34   THEN " + "MEANS(   ABC    ,  XYZ)");
        conds.add(new CountCondition(null, CountCondition.GRATEROREQUAL, 34));
        meanings.add(new Meaning("ABC", "XYZ"));
        ignores.add(null);

        rules.add("IF   FALSE  THEN IGNORE(   PREDICATE    ,  PRICE  )");
        conds.add(new FalseCondition(null));
        meanings.add(null);
        ignores.add(new Ignoration("PRICE", Ignoration.PREDICATE));

        rules.add("BLAH BLAH BLAH");
        conds.add(null);
        meanings.add(null);
        ignores.add(null);

    }

    public void testRule()
    {
        int i = 0;
        for (Iterator itr = rules.iterator(); itr.hasNext();)
        {
            System.out.println("test " + i);
            String ruleContent = (String) itr.next();
            String fileName = makeTempFile(ruleContent);
            Rule rule = null;
            Condition c = (Condition) conds.get(i);
            Meaning m = (Meaning) meanings.get(i);
            Ignoration ign = (Ignoration) ignores.get(i);

            try
            {
                rule = new Rule(fileName);
                assertTrue(compareRule(rule, c, m, ign));
            }
            catch (Exception e)
            {
                e.printStackTrace();
                rule = null;
                assertTrue((c == null) && (m == null) && (ign == null));
            }
            i++;
        }
        File f = new File("test.rule");
        f.delete();
    }

    private boolean compareRule(Rule rule, Condition c, Meaning m, Ignoration ign)
    {
        return rule.compare(c, m, ign);
    }

    private String makeTempFile(String content)
    {
        try
        {
            FileWriter writer = new FileWriter("test.rule");
            writer.write(content);
            writer.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return "test.rule";
    }
}