package com.wawrzek.fumbo.tools.basic;

import java.io.*;
import java.util.*;

public class Record
{
    private Map data;
    private Map names;

    public Record()
    {
        data = new HashMap();
        names = new HashMap();
    }

    public void put(String field, Object value)
    {
        data.put(field, value);
    }

    public void put(String field, String name, Object value)
    {
        data.put(field, value);
        names.put(field, name);
    }

    public void put(String field, int value)
    {
        data.put(field, new Integer(value));
    }

    public void put(String field, String name, int value)
    {
        data.put(field, new Integer(value));
        names.put(field, name);
    }

    public void put(String field, double value)
    {
        data.put(field, new Double(value));
    }

    public void put(String field, String name, double value)
    {
        data.put(field, new Double(value));
        names.put(field, name);
    }

    public int getInt(String field)
    {
        Integer iValue = null;
        try
        {
            iValue = ((Integer) data.get(field));
        }
        catch (ClassCastException ex)
        {
            String strValue = ((String) data.get(field));
            iValue = new Integer(strValue);
        }

        return iValue.intValue();
    }

    public double getDouble(String field)
    {
        Double dValue = null;
        try
        {
            dValue = ((Double) data.get(field));
        }
        catch (ClassCastException ex)
        {
            String strValue = ((String) data.get(field));
            dValue = new Double(strValue);
        }

        return dValue.doubleValue();
    }

    public String getString(String field)
    {
        return data.get(field).toString();
    }

    public Object get(String fieldName)
    {
        return data.get(fieldName);
    }

    public String getName(String fieldCode)
    {
        if (names == null)
            return fieldCode;
        String name = (String) names.get(fieldCode);
        if (name != null)
        {
            return name;
        }
        if (data.containsKey(fieldCode))
        {
            return fieldCode;
        }
        return null;
    }

    public String[] getFields()
    {
        Object[] source = data.keySet().toArray();
        String[] result = new String[source.length];

        for (int i = 0; i < result.length; i++)
            result[i] = (String) (source[i]);
        return result;
    }

    public String toString()
    {
        String result = "Record: ";
        Object[] source = data.keySet().toArray();

        for (int i = 0; i < source.length; i++)
        {
            result += source[i] + "=" + data.get(source[i]) + ",  ";
        }

        return result;
    }

    public boolean read(String fileName)
    {
        try
        {
            BufferedReader br = new BufferedReader(new FileReader(fileName));
            while (true)
            {
                String line = br.readLine();
                if (line == null)
                    break;
                int eqindex = line.indexOf("=");
                String field = line.substring(0, eqindex).trim();
                String value = line.substring(eqindex + 1, line.length()).trim();
                if (value.startsWith("\""))
                {
                    value = value.replaceAll("\"", "");
                    value = value.trim();
                    this.put(field, value);
                }
                else
                {
                    int dotindex = value.indexOf(".");
                    if (dotindex != -1)
                    {
                        double dvalue = new Double(value).doubleValue();
                        this.put(field, dvalue);
                    }
                    else
                    {
                        int ivalue = new Integer(value).intValue();
                        this.put(field, ivalue);
                    }
                }
            }
            br.close();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            return false;
        }
        return true;
    }

}