package com.wawrzek.fumbo.worker;

import jade.content.*;
import jade.content.lang.Codec.*;
import jade.content.onto.*;
import jade.content.onto.basic.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

import org.apache.log4j.*;

import com.wawrzek.fumbo.*;
import com.wawrzek.fumbo.core.*;
import com.wawrzek.fumbo.tools.*;

/**
 * @author Wawrzek
 * 
 * behaviour that expands the result set - does the core job of Fumbo.
 */
public class ExpandResultSetBehaviour extends OneShotBehaviour
{
    
    private ACLMessage question;

    public ExpandResultSetBehaviour(ACLMessage question)
    {
        super();
        this.question = question;
    }

    /* (non-Javadoc)
     * @see jade.core.behaviours.Behaviour#action()
     */
    public void action()
    {
        ContentElement ce = null;
        try
        {
            Action a = (Action) myAgent.getContentManager().extractContent(question);
            Worker worker = (Worker)myAgent;
            Task task = (Task) a.getAction();
            Logger.getLogger(Consts.WORKER_LOG).info(myAgent.getLocalName() + " received a task no. " + task.getId());
            task.expandResultSet(worker.getRule(), worker.getConfig().getString(Consts.FLD_DATAURL), worker.getConfig().getString(Consts.FLD_BASEURI));
            task.increaseRing();

            ACLMessage forward = null;
            if (task.isReady())
            {
                forward = new ACLMessage(ACLMessage.INFORM);
                forward.addReceiver(task.getCoordinator());
            }
            else
            {
                forward = new ACLMessage(ACLMessage.REQUEST);
                forward.addReceiver(task.getNextAgent().getAid());
            }

            forward.setLanguage(((Worker) myAgent).getCodec().getName());
            forward.setOntology(((Worker) myAgent).getOntology().getName());

            a = new Action(myAgent.getAID(), task);
            myAgent.getContentManager().fillContent(forward, a);
            myAgent.send(forward);
            if (task.isReady())
                Logger.getLogger(Consts.WORKER_LOG).info(myAgent.getLocalName() + " forwarded a task no. " + task.getId() + " to " + task.getCoordinator().getLocalName() + ", |MRS|=" + task.getResultsCount());
            else
                Logger.getLogger(Consts.WORKER_LOG).info(myAgent.getLocalName() + " forwarded a task no. " + task.getId() + " to " + task.getNextAgent().getAid().getLocalName() + ", |MRS|=" + task.getResultsCount());

        }

        catch (UngroundedException e)
        {
            Logger.getLogger(Consts.WORKER_LOG).error("Ungroundeds error: " + e.getMessage());
            e.printStackTrace();
        }
        catch (CodecException e)
        {
            Logger.getLogger(Consts.WORKER_LOG).error("codec error: " + e.getMessage());
            e.printStackTrace();
            e.getNested().printStackTrace();
        }
        catch (OntologyException e)
        {
            Logger.getLogger(Consts.WORKER_LOG).error("Ontology error: " + e.getMessage());
            e.printStackTrace();
        }
        catch (AgentException e)
        {
            Logger.getLogger(Consts.WORKER_LOG).error("Cannot expand result set: " + e.getMessage());
            e.printStackTrace();
        }
    }
}